@@ -319,7 +319,9 @@ get_property (GObject *object, guint prop_id,
  * @optname1: name of first property to set (or %NULL)
  * @...: value of @optname1, followed by additional property/value pairs
  *
- * Return value: a new (disconnected) socket
+ * Creates a new (disconnected) socket
+ *
+ * Return value: the new socket
  **/
 SoupSocket *
 soup_socket_new (const char *optname1, ...)
@@ -597,6 +599,22 @@ soup_socket_listen (SoupSocket *sock, SoupAddress *local_addr)
  **/
 gboolean
 soup_socket_start_ssl (SoupSocket *sock)
+{
+	return soup_socket_start_proxy_ssl (sock, soup_address_get_name (sock->priv->remote_addr));
+}
+	
+/**
+ * soup_socket_start_proxy_ssl:
+ * @sock: the socket
+ * @ssl_host: hostname of the SSL server
+ *
+ * Starts using SSL on @socket, expecting to find a host named
+ * @ssl_host.
+ *
+ * Return value: success or failure
+ **/
+gboolean
+soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host)
 {
 	GIOChannel *ssl_chan;
 
@@ -604,8 +622,7 @@ soup_socket_start_ssl (SoupSocket *sock)
 	ssl_chan = soup_ssl_wrap_iochannel (
 		sock->priv->iochannel, sock->priv->is_server ?
 		SOUP_SSL_TYPE_SERVER : SOUP_SSL_TYPE_CLIENT,
-		soup_address_get_name (sock->priv->remote_addr),
-		sock->priv->ssl_creds);
+		ssl_host, sock->priv->ssl_creds);
 
 	if (!ssl_chan)
 		return FALSE;
@@ -727,6 +744,13 @@ soup_socket_server_new (SoupAddress *local_addr, gpointer ssl_creds,
 }
 
 
+/**
+ * soup_socket_disconnect:
+ * @sock: a #SoupSocket
+ *
+ * Disconnects @sock. Any further read or write attempts on it will
+ * fail.
+ **/
 void
 soup_socket_disconnect (SoupSocket *sock)
 {
@@ -771,6 +795,14 @@ soup_socket_disconnect (SoupSocket *sock)
 	g_signal_emit (sock, signals[DISCONNECTED], 0);
 }
 
+/**
+ * soup_socket_is_connected:
+ * @sock: a #SoupSocket
+ *
+ * Tests if @sock is connected to another host
+ *
+ * Return value: %TRUE or %FALSE.
+ **/
 gboolean
 soup_socket_is_connected (SoupSocket *sock)
 {
@@ -779,7 +811,14 @@ soup_socket_is_connected (SoupSocket *sock)
 	return sock->priv->iochannel != NULL;
 }
 
-
+/**
+ * soup_socket_get_local_address:
+ * @sock: a #SoupSocket
+ *
+ * Returns the #SoupAddress corresponding to the local end of @sock.
+ *
+ * Return value: the #SoupAddress
+ **/
 SoupAddress *
 soup_socket_get_local_address (SoupSocket *sock)
 {
@@ -799,6 +838,14 @@ soup_socket_get_local_address (SoupSocket *sock)
 	return sock->priv->local_addr;
 }
 
+/**
+ * soup_socket_get_remote_address:
+ * @sock: a #SoupSocket
+ *
+ * Returns the #SoupAddress corresponding to the remote end of @sock.
+ *
+ * Return value: the #SoupAddress
+ **/
 SoupAddress *
 soup_socket_get_remote_address (SoupSocket *sock)
 {
@@ -848,7 +895,13 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len, gsize *nread)
 		if (err->domain == SOUP_SSL_ERROR &&
 		    err->code == SOUP_SSL_ERROR_HANDSHAKE_NEEDS_WRITE)
 			cond = G_IO_OUT;
-		g_error_free (err);
+		g_object_set_data_full (G_OBJECT (sock),
+					"SoupSocket-last_error",
+					err, (GDestroyNotify)g_error_free);
+	} else {
+		g_object_set_data (G_OBJECT (sock),
+				   "SoupSocket-last_error",
+				   NULL);
 	}
 
 	switch (status) {
@@ -1068,7 +1121,13 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 		if (err->domain == SOUP_SSL_ERROR &&
 		    err->code == SOUP_SSL_ERROR_HANDSHAKE_NEEDS_READ)
 			cond = G_IO_IN;
-		g_error_free (err);
+		g_object_set_data_full (G_OBJECT (sock),
+					"SoupSocket-last_error",
+					err, (GDestroyNotify)g_error_free);
+	} else {
+		g_object_set_data (G_OBJECT (sock),
+				   "SoupSocket-last_error",
+				   NULL);
 	}
 
 	if (status != G_IO_STATUS_NORMAL && status != G_IO_STATUS_AGAIN) {
